/*
 * Generic DMA support.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/cpm-generic.h>
#include <l4/re/c/dma_space.h>

// DMA channel.

class Dma_channel
{
public:
  virtual unsigned int transfer(uint32_t source, uint32_t destination,
                                unsigned int count,
                                bool source_increment, bool destination_increment,
                                uint8_t source_width, uint8_t destination_width,
                                uint8_t transfer_unit_size,
                                int type,
                                l4_addr_t desc_vaddr = 0,
                                l4re_dma_space_dma_addr_t desc_paddr = 0) = 0;

  virtual unsigned int wait() = 0;
};

// DMA device control.

class Dma_chip
{
public:
  virtual void disable() = 0;

  virtual void enable() = 0;

  virtual Dma_channel *get_channel(uint8_t channel, l4_cap_idx_t irq) = 0;

  // Transaction control.

  virtual void ack_irq(uint8_t channel)
  {
    (void) channel;
  }

  virtual bool error()
  {
    return false;
  }

  virtual bool halted()
  {
    return false;
  }

  virtual void commit_descriptor(uint8_t channel)
  {
    (void) channel;
  }

  virtual bool have_interrupt(uint8_t channel) = 0;
};

// Generic access to the DMA abstraction.

Dma_chip *new_dma_chip(const char *name, l4_addr_t start, l4_addr_t end, Cpm_chip *cpm);

#endif /* __cplusplus */
